﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using JsMapper.BL.Config;
using JsMapper.BO;
using JsMapper.BO.Events;

#endregion

namespace JsMapper.BL
{
    public static class JsModelManager
    {
        static JsModelManager()
        {
            Repository = new JsModelRepository();
            ConfigMngr = new JsModelConfigManager();
            LoadedMappers = new List<IJsModelMapper>();

            ConfigMngr.OnCollection += OnJsModelCollection;
            ConfigMngr.OnUseCache += OnUseCache;
        }

        public static JsModelRepository Repository { get; set; }
        public static JsModelConfigManager ConfigMngr { get; set; }

        private static Dictionary<Type, Dictionary<Type, string>> JsModelsCache { get; set; }
        private static Dictionary<Type, string> AllJsModelsCache { get; set; }
        private static List<IJsModelMapper> LoadedMappers { get; set; }

        private static void OnJsModelCollection(object sender, JsModelCollectionEventArgs args)
        {
            Repository.AddRange(args.JsModelCollection);

            if (JsModelsCache != null) JsModelsCache.Clear();
            if (AllJsModelsCache != null) AllJsModelsCache.Clear();
        }

        private static void OnUseCache(object sender, JsModelCacheEventArgs args)
        {
            JsModelsCache = new Dictionary<Type, Dictionary<Type, string>>();
            AllJsModelsCache = new Dictionary<Type, string>();
        }

        private static T GetMapper<T>() where T : class, IJsModelMapper
        {
            var mapper = LoadedMappers.OfType<T>().FirstOrDefault();
            if (mapper != null) return mapper;

            var mapperInst = Activator.CreateInstance<T>();
            LoadedMappers.Add(mapperInst);

            return mapperInst;
        }

        public static string GetJsOutput<T>(Type modelType) where T : class, IJsModelMapper
        {
            if (ConfigMngr.Config.UseCache && JsModelsCache.ContainsKey(typeof (T)) &&
                JsModelsCache[typeof (T)].ContainsKey(modelType)) return JsModelsCache[typeof (T)][modelType];

            var mapper = GetMapper<T>();
            var jsModel = Repository.Get(modelType);
            if (jsModel == null) jsModel = JsModelBuilder.Create<T>();

            var output = JsOutputBuilder.ForJsModel(jsModel, ConfigMngr.Config, mapper);

            if (ConfigMngr.Config.UseCache)
            {
                if (!JsModelsCache.ContainsKey(typeof (T)))
                    JsModelsCache.Add(typeof (T), new Dictionary<Type, string>());

                JsModelsCache[typeof (T)].Add(modelType, output);
            }

            return output;
        }

        public static string GetJsOutput<TY, T>()
            where T : class, IJsModelMapper
            where TY : class
        {
            return GetJsOutput<T>(typeof (TY));
        }

        public static string GetAllJsOutput<T>() where T : class, IJsModelMapper
        {
            if (ConfigMngr.Config.UseCache &&
                AllJsModelsCache.ContainsKey(typeof (T)) &&
                !string.IsNullOrEmpty(AllJsModelsCache[typeof (T)])) return AllJsModelsCache[typeof (T)];

            var models = Repository.GetAll()
                                   .Select(m => m.ModelType)
                                   .Aggregate(string.Empty, (current, t) => current + GetJsOutput<T>(t));

            if (ConfigMngr.Config.UseCache && !AllJsModelsCache.ContainsKey(typeof (T)))
                AllJsModelsCache.Add(typeof (T), models);
            return models;
        }
    }
}